C**********************************************************************
C                 Copyright 1992-1996, Robert Fraczkiewicz            *
C                           All rights reserved.                      *
C**********************************************************************
C This program is free software; you can redistribute it and/or modify*
C it under the terms of the GNU General Public License as published by*
C the Free Software Foundation; either version 1, or (at your option) *
C any later version.                                                  *
C                                                                     *
C This program is distributed in the hope that it will be useful,     *
C but WITHOUT ANY WARRANTY; without even the implied warranty of      *
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU    *
C General Public License for more details.                            *
C                                                                     *
C You should  have received a copy of the GNU General Public License  *
C along with this program; if not, write to the Free Software         *
C Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.           *
c*********************************************************************
c                                                                    *
c          Program "Rank of F-Matrix" by Robert Fraczkiewicz         *
c                        Jun 1992, version 1.0                       *
c                                                                    *
c*********************************************************************
c Program calculates the rank of a F-matrix block. The Z-matrix is   *
c read from a binary ".zmat" file; the force field is supplied in a  *
c ".fi" file generated by FR or by a user.                           *
c*********************************************************************
c                                                                    *
c      INPUT FILES:                                                  *
c                                                                    *
c      ".zmat" - Z-matrix block in binary format (ZSYM4 output)      *
c      ".fi" - force field (output of FR or REF6)                    *
c                                                                    *
c*********************************************************************
c                                                                    *
c      OUTPUT FILES:                                                 *
c                                                                    *
c      ".raoutf" - contains rank value, eigenvalues and eigenvectors *
c                                                                    *
c*********************************************************************
c                                                                    *
c      DIMENSIONS:                                                   *
c                                                                    *
c      nr,nc,z,nfo                              - NDIMZ              *
c      ak                                       - INTERxINTER        *
c      w                                        - INTER              *
c      work                                     - LWORK              *
c                                                                    *
c*********************************************************************
c                                                                    *
c     BLAS AND LAPACK ROUTINES:                                      *
c                                                                    *
c     dsyev - eigenanalysis of a real symmetric matrix               * 
c                                                                    *
c On SGI machines LAPACK library is specified by f77 compiler option *
c -lcomplib.sgimath                                                  *
c                                                                    *
c*********************************************************************
      program rankf
      implicit real*8(a-h,o-z)
      include 'Sizes'
      dimension nr(ndimz),nc(ndimz),z(ndimz),nfo(ndimz),
     1 ak(inter,inter),w(inter),work(lwork)
      character*60 name,name1,cr
c**********************************************************************
      if(iargc().lt.4) then 
       print *
       print *,' /--------------------------------------------------\ '
       print *,'|                                                    |'
       print *,'|                    R  A  N  K  F                   |'
       print *,'|                                                    |'
       print *,'|                     version 1.0                    |'
       print *,'|                                                    |'
       print *,'|                 Rank of the F-Matrix               |'
       print *,'|                                                    |'
       print *,'|                           by Robert Fraczkiewicz   |'
       print *,'|                      University of Houston, 1992   |'
       print *,' \--------------------------------------------------/ '
       print *
       print *,'______________________________________________________'
       print *
       print *,'SYNOPSIS: '
       print *
       print *,'      rankf filename1 filename2 nq nf'
       print *
       print *,'PARAMETERS: '
       print *
       print *,'      filename1 - core name of the ".zmat" file'
       print *
       print *,'      filename2 - core name of the ".fi" file'
       print *,'      (Appropriate extensions are added by default).'
       print *
       print *,'      nq - dimension of the F-matrix'
       print *
       print *,'      nf - number of force constants'
       print *
       print *,'______________________________________________________'
       stop 7
      endif
      narg=1
      call getarg(narg,name)
      namlen = lstnbl(name)
      narg=narg+1
      call getarg(narg,name1)
      namlen1 = lstnbl(name1)
      narg=narg+1
      call getarg(narg,cr)
      read(cr,*) nq
      narg=narg+1
      call getarg(narg,cr)
      read(cr,*) nf
      nrank=nq
c**************Read force constants*****************************
      open(unit=3,file=name1(1:namlen1)//'.fi', status='old', 
     .     err=9991)     
      print *,'Reading '//name1(1:namlen1)//'.fi'
      read(3,*) (w(i),i=1,nf)
      close(unit=3)
c**************Read a Z matrix block**********************************
c*Z matrix must be ordered in the descending order of force constants*
c*********************************************************************
      open(unit=7,file=name(1:namlen)//'.zmat', status='old',
     .     form='unformatted', err=9991)
      print *,'Reading '//name(1:namlen)//'.zmat'
      read(7) noz
      read(7)(nr(l),nc(l),nfo(l),z(l),l=1,noz)
      close(unit=7)
      do i=1,nq
       do j=1,nq
        ak(i,j)=0.0d0
       enddo
      enddo
      do 110 l=1,noz
       i=nr(l)
       j=nc(l)
  110  ak(i,j)=ak(i,j)+z(l)*w(nfo(l))
      call dsyev('V','U',nq,ak,inter,w,work,lwork,info)
      n=nrank
      do i=1,n
       if(abs(w(i)).lt.1.0d-04) nrank=nrank-1
      enddo
      open(unit=16, file=name(1:namlen)//'.raoutf', status='unknown')
      write(16,45) name(1:namlen),nrank
   45 format(1x,'File : ',a14,/1x,'Rank of the F-matrix = ',i3)
      write(16,43)
   43 format(/1x,'Eigenvalues'//)
      do 80 k=1,n
   80  write(16,44) k,k,w(k)
   44 format(1x,'a(',i3,',',i3,')=',e13.6)
      write(16,48)
   48 format(/1x,'Eigenvectors'//)
      do i=1,n
       write(16,46) i,w(i)
       write(16,47) (j,ak(j,i), j=1,n)
      enddo
   46 format(1x,i3,' - th eigenvalue = ',e13.6)
   47 format(6(1x,i3,1x,f7.4))
      close(unit=16)
      stop 1
c  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 9991 print *
      print *,'Input operation unsuccesful. Cannot find input file'
      stop 3
c  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      end

c  -------------------  lstnbl -----
c  routine which finds last nonblank character in a character variable
c
      integer function lstnbl(chrvar)
      character*(*) chrvar
      integer i, l
      l = len(chrvar)
      do 100 i = l, 1, -1
        if(chrvar(i:i) .ne. ' ')goto 200
100   continue
200   lstnbl = i
      return
      end

