C**********************************************************************
C                 Copyright 1992-1996, Robert Fraczkiewicz            *
C                           All rights reserved.                      *
C**********************************************************************
C This program is free software; you can redistribute it and/or modify*
C it under the terms of the GNU General Public License as published by*
C the Free Software Foundation; either version 1, or (at your option) *
C any later version.                                                  *
C                                                                     *
C This program is distributed in the hope that it will be useful,     *
C but WITHOUT ANY WARRANTY; without even the implied warranty of      *
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU    *
C General Public License for more details.                            *
C                                                                     *
C You should  have received a copy of the GNU General Public License  *
C along with this program; if not, write to the Free Software         *
C Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.           *
c*********************************************************************
c                                                                    *
c          Program "Rank of G-Matrix" by Robert Fraczkiewicz         *
c                        Jan 1993, version 2.0                       *
c                                                                    *
c*********************************************************************
c Program calculates the rank of a G-matrix block. The G-matrix is   *
c read from a binary ".gmat" file.                                   *
c*********************************************************************
c                                                                    *
c      INPUT FILES:                                                  *
c                                                                    *
c      ".gmat" - G-matrix block in binary format (GMAT4 output)      *
c                                                                    *
c*********************************************************************
c                                                                    *
c      OUTPUT FILES:                                                 *
c                                                                    *
c      ".raoutg" - contains rank value, eigenvalues and eigenvectors *
c                                                                    *
c*********************************************************************
c                                                                    *
c      DIMENSIONS:                                                   *
c                                                                    *
c      nr,nc,z,nfo                              - NDIMZ              *
c      ak                                       - INTERxINTER        *
c      w                                        - INTER              *
c      work                                     - LWORK              *
c                                                                    *
c*********************************************************************
c                                                                    *
c     BLAS AND LAPACK ROUTINES:                                      *
c                                                                    *
c     dsyev - eigenanalysis of a real symmetric matrix               * 
c                                                                    *
c On SGI machines LAPACK library is specified by f77 compiler option *
c -lcomplib.sgimath                                                  *
c                                                                    *
c*********************************************************************
      program rankg
      implicit real*8(a-h,o-z)
      include 'Sizes'
      dimension nr(ndimz),nc(ndimz),z(ndimz),ak(inter,inter),
     1 w(inter),work(lwork)
      character*60 name,cr
c**********************************************************************
      if(iargc().lt.2) then 
       print *
       print *,' /--------------------------------------------------\ '
       print *,'|                                                    |'
       print *,'|                    R  A  N  K  G                   |'
       print *,'|                                                    |'
       print *,'|                     version 2.0                    |'
       print *,'|                                                    |'
       print *,'|                 Rank of the G-Matrix               |'
       print *,'|                                                    |'
       print *,'|                           by Robert Fraczkiewicz   |'
       print *,'|                      University of Houston, 1993   |'
       print *,' \--------------------------------------------------/ '
       print *
       print *,'______________________________________________________'
       print *
       print *,'SYNOPSIS: '
       print *
       print *,'      rankg filename nq'
       print *
       print *,'PARAMETERS: '
       print *
       print *,'      filename - core name of the ".gmat" file'
       print *,'      (Appropriate extensions are added by default).'
       print *
       print *,'      nq - dimension of the G-matrix block'
       print *
       print *,'______________________________________________________'
       stop 7
      endif
      narg=1
      call getarg(narg,name)
      namlen = lstnbl(name)
      narg=narg+1
      call getarg(narg,cr)
      read(cr,*) nq
      nrank=nq
      noz=0
c***************read g matrix******************************
      print *,'Reading '//name(1:namlen)//'.gmat'
      open(unit=7,file=name(1:namlen)//'.gmat',status='old',form
     . ='unformatted', err=9991)
      read(7) noz
      read(7) (nr(l),nc(l),z(l),l=1,noz)   
      close(unit=7)
      do i=1,nq
       do j=1,nq
        ak(i,j)=0.0d0
       enddo
      enddo
      do l=1,noz
       i=nr(l)
       j=nc(l)
       ak(i,j)=ak(i,j)+z(l)
       if(i.ne.j) ak(j,i)=ak(i,j)
      enddo
      call dsyev('V','U',nq,ak,inter,w,work,lwork,info)
      n=nrank
      do i=1,n
       if(abs(w(i)).lt.1.0d-04) nrank=nrank-1
      enddo
      open(unit=16, file=name(1:namlen)//'.raoutg', status='unknown')
      write(16,45) name(1:namlen),nrank
   45 format(1x,'File : ',a14,/1x,'Rank of the G-matrix = ',i3)
      write(16,43)
   43 format(/1x,'Eigenvalues'//)
      do 80 k=1,n
   80  write(16,44) k,k,w(k)
   44 format(1x,'a(',i3,',',i3,')=',e13.6)
      write(16,48)
   48 format(/1x,'Eigenvectors'//)
      do i=1,n
       write(16,46) i,w(i)
       write(16,47) (j,ak(j,i), j=1,n)
      enddo
   46 format(1x,i3,' - th eigenvalue = ',e13.6)
   47 format(6(1x,i3,1x,f7.4))
      close(unit=16)
      stop 1
c  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 9991 print *
      print *,'Input operation unsuccesful. Cannot find input file'
      stop 3
c  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      end

c  -------------------  lstnbl -----
c  routine which finds last nonblank character in a character variable
c
      integer function lstnbl(chrvar)
      character*(*) chrvar
      integer i, l
      l = len(chrvar)
      do 100 i = l, 1, -1
        if(chrvar(i:i) .ne. ' ')goto 200
100   continue
200   lstnbl = i
      return
      end
