C**********************************************************************
C                 Copyright 1992-1996, Robert Fraczkiewicz            *
C                           All rights reserved.                      *
C**********************************************************************
C This program is free software; you can redistribute it and/or modify*
C it under the terms of the GNU General Public License as published by*
C the Free Software Foundation; either version 1, or (at your option) *
C any later version.                                                  *
C                                                                     *
C This program is distributed in the hope that it will be useful,     *
C but WITHOUT ANY WARRANTY; without even the implied warranty of      *
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU    *
C General Public License for more details.                            *
C                                                                     *
C You should  have received a copy of the GNU General Public License  *
C along with this program; if not, write to the Free Software         *
C Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.           *
c*********************************************************************
c                                                                    *
c         Program "Rank of Z-matrix" by Robert Fraczkiewicz          *
c                         Feb 1992, version 1.1                      *
c                                                                    *
c*********************************************************************
c Program calculates the rank of a Z-matrix block or combination of  *
c blocks read from binary ".zmat" file(s). For the reason of Z-matrix*
c simplicity, the Gaussian elimination method is used to find all    *
c linear dependencies among force constants.                         *
c*********************************************************************
c                                                                    *
c      INPUT FILES:                                                  *
c                                                                    *
c      ".zmat" - Z-matrix block in binary format (ZSYM4 output)      *
c                                                                    *
c*********************************************************************
c                                                                    *
c      OUTPUT FILES:                                                 *
c                                                                    *
c      ".raout" - contains rank value and linear dependence relations*
c                                                                    *
c*********************************************************************
c                                                                    *
c      DIMENSIONS:                                                   *
c                                                                    *
c      nr,nc,z,nfo                              - NDIMZ              *
c      ak,nrk                                   - NFCxNONZERO        *
c      nonz                                     - NFC                *
c                                                                    *
c*********************************************************************
      implicit real*8(a-h,o-z)
      include 'Sizes'
      integer nonzero
      parameter(nonzero=300)
      dimension nr(ndimz),nc(ndimz),nfo(ndimz),z(ndimz),
     .nrk(nfc,nonzero),ak(nfc,nonzero),nonz(nfc)
      logical all
      character*60 name,cr,crz
c**********************************************************************
      nn=iargc()
      if(iargc().ne.2) then
       print *,'^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^'
       print *
       print *,'                      R A N K Z    '
       print *
       print *,'                     Version 1.1'
       print *
       print *,'              Rank of a combined Z-matrix '
       print *
       print *,'______________________________________________________'
       print *
       print *,'SYNOPSIS:'
       print *
       print *,'     rankz nblo nf'
       print *
       print *,'PARAMETERS:'
       print *
       print *,'   nblo : number of blocks to be examined'
       print *,'   nf   : total number of independent force constants '
       print *
       print *,'______________________________________________________'
       stop 1
      endif
      call getarg(1,cr)
      read(cr,*) nblo
      call getarg(2,cr)
      read(cr,*) nf
      nrank=nf
      all=.false.
      nzo=0
      nq=0
c**************Read force constants*****************************
      do i=1,ndimz
       nfo(i)=0
      enddo
      do inbl=1,nblo
       print *,'Block : ',inbl
       print *
       print *,'Input the name of the file (without extension !!!)     '
       print *,'where the Z-matrix is stored and press <return>.       '
       print *,'The ".zmat" extension will be automatically assumed.   '
       read(*,'(a60)') name
       namlen=lstnbl(name)
       print *,'Input the dimension of the Z-matrix'
       read(*,*) nnq
       print *
       print *
c**************Read a Z matrix block**********************************
c*Z matrix must be ordered in the descending order of force constants*
c*********************************************************************
       open(unit=7, file=name(1:namlen)//'.zmat', err=9991,
     .     status='old',form='unformatted')
       read(7) noz
       read(7)(nr(nzo+l),nc(nzo+l),nfo(nzo+l),z(nzo+l),l=1,noz)
       close(unit=7)
       do l=1,noz
        nzl=nzo+l
        nr(nzl)=nq+nr(nzl)
        nc(nzl)=nq+nc(nzl)
       enddo
       nzo=nzo+noz
       nq=nq+nnq
      enddo
c*****************Prepare matrices************************
      print *,'Results will be stored in '//name(1:namlen)//'.raout'
      open(unit=16, file=name(1:namlen)//'.raout',
     1 status='unknown') 
      do i=1,nfc
       nonz(i)=0
       do j=1,nonzero
        nrk(i,j)=0
        ak(i,j)=0.0
       enddo
      enddo
      do l=1,nzo
       k=nfo(l)
       nonz(k)=nonz(k)+1
       nrk(k,nonz(k))=nq*(nr(l)-1)+nc(l)
       ak(k,nonz(k))=z(l)
      enddo
c*******************Start Gaussian elimination*************
      do 80 k=1,nf
       if(nonz(k)) 80,90,100
   90  nrank=nrank-1
       write(16,41) k
   41  format(1x,i3,' - th force constant not present')
       goto 80
  100  kol=1
       amax=ak(k,1)
       do j=2,nonz(k)
        if(abs(ak(k,j)).gt.abs(amax)) then
         amax=ak(k,j)
         kol=j
        endif
       enddo
       if(amax.ne.0.0) then
        kk=nrk(k,kol)
        if(kk.ne.k) then
         do l=1,nf
          do ii=1,nonz(l)
           kx=nrk(l,ii)
           if(kx.eq.kk) nrk(l,ii)=k
           if(kx.eq.k) nrk(l,ii)=kk
          enddo
         enddo
        endif
        do ii=k+1,nf
         do m=1,nonz(ii)
          if(nrk(ii,m).eq.k) then
           all=.true.
           r=ak(ii,m)
           do n=1,nonz(ii)
            do j=1,nonz(k)
             if(nrk(ii,n).eq.nrk(k,j)) then
              ak(ii,n)=ak(ii,n)-ak(k,j)*r/amax
              if(abs(ak(ii,n)).lt.9.0e-07) then
               ak(ii,n)=0.0
              else
               all=.false.
              endif
             endif
            enddo
           enddo
          endif
         enddo
         if(all) then
          nrank=nrank-1
          nonz(ii)=-1
          if(r.gt.0.0) then
           write(16,42) amax,k,r,ii
          else
           write(16,43) amax,k,r,ii
          endif             
         endif
         all=.false.
        enddo
       endif
   80 continue
      write(16,45) name,nrank
   42 format(1x,'Linear dependence: ',f7.4,'*k(',i3,')+',f6.4,'*k(',
     1i3,')')
   43 format(1x,'Linear dependence: ',f7.4,'*k(',i3,')',f7.4,'*k(',
     1i3,')')
   45 format(1x,'File : ',a14,/1x,'Rank of the combined Z-matrix = ',i3)
      close(unit=16)
      stop 1
c  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 9991 print *
      print *,
     1 'Input operation unsuccesful. Cannot find Z-matrix file '//
     2 name(1:namlen)//'.zmat'
      stop 3
c  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      end

c  -------------------  lstnbl -----
c  routine which finds last nonblank character in a character variable
c
      integer function lstnbl(chrvar)
      character*(*) chrvar
      integer i,l
      l=len(chrvar)
      do i=l,1,-1
       if(chrvar(i:i).ne.' ')goto 200
      enddo
200   lstnbl=i
      return
      end
