C**********************************************************************
C                 Copyright 1992-1996, Robert Fraczkiewicz            *
C                           All rights reserved.                      *
C**********************************************************************
C This program is free software; you can redistribute it and/or modify*
C it under the terms of the GNU General Public License as published by*
C the Free Software Foundation; either version 1, or (at your option) *
C any later version.                                                  *
C                                                                     *
C This program is distributed in the hope that it will be useful,     *
C but WITHOUT ANY WARRANTY; without even the implied warranty of      *
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU    *
C General Public License for more details.                            *
C                                                                     *
C You should  have received a copy of the GNU General Public License  *
C along with this program; if not, write to the Free Software         *
C Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.           *
c**********************************************************************
c                                                                     *
c     Program "Force Constant Selection" by Robert Fraczkiewicz       *
c                       Jun 95, version 2.6                           *
c                                                                     *
c**********************************************************************
c The program uses fitting potential of an adjustable set to select   *
c optimal force constants for refinement. Selection is based on a     *
c novel method of modified regularized least-squares developed by     *
c author. SEL checks an effect of addition of each individual force   *
c constant out of the fixed set. The ".rein" file usually contains the*
c fixed set completion (i.e., adjustable set). The fixed set can be   *
c alternatively defined in a ".sel" file. SEL can additionally check  *
c a result of adjustable force constant elimination. Please, refer to *
c literature for further details.                                     *
c**********************************************************************
c                                                                     *
c     INPUT FILES:                                                    *
c                                                                     *
c     ".rein" - main input file containing force field and obseved    *
c               vibrational frequencies                               *
c     ".zmat" - Z-matrix (output of ZSYM4)                            *
c     ".gmat" - G-matrix (output of GMAT4)                            *
c     ".sel" - optional set of fixed force constants                  *
c                                                                     *
c**********************************************************************
c                                                                     *
c     OUTPUT FILES:                                                   *
c                                                                     *
c     ".selout" - results of SEL: force constant vs. unrefinable part *
c                 of the b-vector                                     *
c                                                                     *
c**********************************************************************
c                                                                     *
c     BLAS AND LAPACK ROUTINES:                                       *
c                                                                     *
c     dsymm - symmetric matrix product                                *
c     dgemm - general matrix product                                  *
c     dgemv - general matrix-vector product                           *
c     dspev - eigenanalysis of real symmetric matrix in packed storage*
c     dgesvd - singular value decomposition of a real general marix   *
c     dsyev - eigenanalysis of a real symmetric matrix                *         c     dsyrk - symmetric matrix update                                 *
c                                                                     *
c On SGI machines LAPACK library is specified by f77 compiler option  *
c -lcomplib.sgimath                                                   *
c                                                                     *
c**********************************************************************
c                                                                     *
c     DIMENSIONS:                                                     *
c                                                                     *
c     work                           - LWORK                          *
c     nfo,nr,nc,z                    - NBLxNDIMZ                      *
c     ndown,nup                      - NBLxNFC                        *
c     q                              - NDIMxNDIMxNBL                  *
c     h0                             - [NDIM*(NDIM+1)/2]xNBL          *
c     nq,nz,noz,nfq                  - NBL                            *
c     fq,eig,nzfq                    - NBLxNDIM                       *
c     fi,a,fi0,ncc1,ncc,nfix         - NFC                            *
c     g,w,opt                        - NFCxNFC                        *
c     b                              - NBL*NDIM                       *
c     aa,u                           - NBL*NDIMxNFC                   *
c                                                                     *
c**********************************************************************

      program sel
      implicit real*8(a-h,o-z)
      include 'Sizes'
      parameter(inp=2)
      character*20 name,cr
      character*80 comment(nbl)
      logical change,tmpin,sing,elimin,ve
      dimension work(lwork),a(nfc),w(nfc,nfc),g(nfc,nfc),opt(nfc,nfc),
     .          ncc1(nfc),noz(nbl),nq(nbl)
      common /jacob/ aa(jarow,nfc),u(jarow,nfc),ncc(nfc),nfix(nfc)
     . /qmat/ q(ndim,ndim,nbl),fq(nbl,ndim),nfq(nbl),nz(nbl),
     .        nzfq(nbl,ndim),nfo(nbl,ndimz)
     . /hmat/ h0(nkdim,nbl),b(jarow),fi(nfc),fi0(nfc)
     . /hkmatrices/ nr(nbl,ndimz),nc(nbl,ndimz),z(nbl,ndimz),
     . ndown(nbl,nfc),nup(nbl,nfc)
c**********************************************************************
      if(iargc().eq.0) then 
       print *
       print *,' /--------------------------------------------------\ '
       print *,'|            SSSSSSS   EEEEEEEEE  L                  |'
       print *,'|           S       S  E          L                  |'
       print *,'|           S          E          L                  |'
       print *,'|            SSSSSSS   EEEEEEE    L                  |'
       print *,'|                   S  E          L                  |'
       print *,'|           S       S  E          L                  |'
       print *,'|            SSSSSSS   EEEEEEEEE  LLLLLLLLL          |'
       print *,'|                                                    |'
       print *,'|                     version 2.6                    |'
       print *,'|                                                    |'
       print *,'|          Force Constant Selection Program          |'
       print *,'|                                                    |'
       print *,'|                                                    |'
       print *,'|                           by Robert Fraczkiewicz   |'
       print *,'|                      University of Houston, 1992-4 |'
       print *,' \--------------------------------------------------/ '
       print *
       print *,'______________________________________________________'
       print *
       print *,'SYNOPSIS: '
       print *
       print *,'     sel [-e] [-v] filename1 [filename2]'
       print *
       print *,'PARAMETERS: '
       print *
       print *,'      filename1 - core name of the input and output'
       print *,'                  files. Its length cannot exceed 14'
       print *,'                  characters. Appropriate extensions'
       print *,'                  are added by default.'
       print *
       print *,'      filename2 - core name of the optional file'
       print *,'                  containing fixed set of force'
       print *,'                  constants. ".sel" extensions'
       print *,'                  is added by default.'
       print *
       print *,'DESCRIPTION:'
       print *
       print *,' The program uses fitting potential of an adjustable'
       print *,' set to select optimal force constants for refinement.'
       print *,' Selection is based on a novel method of modified '
       print *,' regularized least-squares  developed by author. SEL'
       print *,' checks an effect of addition of each individual force'
       print *,' constant out of the fixed set. The ".rein" file '
       print *,' contains the fixed set completion (i.e., adjustable'
       print *,' set). The fixed  set can be alternatively defined in'
       print *,' a ".sel" file. SEL can additionally check a result of'
       print *,' adjustable force constant elimination. Please, refer'
       print *,' to literature for further details.'
       print *
       print *,'OPTIONS:'
       print *
       print *,'  -e  sel calculates also an effect of elimination'
       print *,'      of current force constants.'
       print *      
       print *,'  -v  verbose flag. Program will produce more output'
       print *,'      to the screen.'
       print *
       print *,'______________________________________________________'
       stop 7
      endif
      cr=''
      elimin=.true.
      ve=.false.
      narg=1
      call getarg(narg,name)
      narg=narg+1
    5 if(name.eq.'-e') then 
       elimin=.false.
       call getarg(narg,name)
       narg=narg+1
      endif
      if(name.eq.'-v') then
       ve=.true.
       call getarg(narg,name)
       narg=narg+1  
       goto 5
      endif
      l_name=namlen(name)
      call getarg(narg,cr)
      open(unit=9,file=name(1:l_name)//'.rein',err=9990,status='old')
c**************Read initial set of force constants*****************
      print *,'Reading '//name(1:l_name)//'.rein'
      read(9,*) nf
      read(9,*) (fi(i),i=1,nf)
      read(9,*) nvar
      if(nvar.lt.0) then
       nvar=nf+nvar
       nfixed=nf-nvar
       read(9,*) (nfix(i),i=1,nfixed)
       l=1
       j=1
       i=0
       do while(l.le.nfixed.or.j.le.nvar)
        i=i+1
        if(i.eq.nfix(l)) then
         l=l+1
        else
         ncc(j)=i
         j=j+1
        endif
       enddo
       do i=1,nvar
        ncc1(i)=ncc(i)
       enddo
      else
       nfixed=nf-nvar
       if(nfixed.eq.0) then
        do i=1,nvar
         ncc(i)=i
         ncc1(i)=i
        enddo
       else
        read(9,*) (ncc(i), i=1,nvar)
        do i=1,nvar
         ncc1(i)=ncc(i)
        enddo
        l=1
        j=1
        i=0
        do while(l.le.nvar.or.j.le.nfixed)
         i=i+1
         if(i.eq.ncc(l)) then
          l=l+1
         else
          nfix(j)=i
          j=j+1
         endif
        enddo
       endif
      endif 
      nvar1=nvar
      nfixed1=nfixed
c************Optionally read fixed set from a file***************
      if(cr.ne.'') then
       l_cr=namlen(cr)
       open(unit=2,file=cr(1:l_cr)//'.sel',err=9990,status='old')
       read(2,*) nfixed1
       read(2,*) (nfix(i), i=1,nfixed1)
       close(unit=2)
      endif
      read(9,*) nb
      f=0
      nroo=0
      rewind 1
      do i=1,nb
c**************Read experimental frequencies*******************
c nblok - number of symmetry block, nmol - number of molecule *
c**************************************************************
       read(9,'(i2,1x,i2,1x,a80)') nblok,nmol,comment(i)
       read(9,*) nfq(i),nq(i)
       read(9,*) (fq(i,j), j=1,nfq(i))
       kk=0
       do j=1,nfq(i)
        if(fq(i,j).eq.0.0) then 
         kk=kk+1
         nzfq(i,kk)=j
        endif
        fq(i,j)=factor*fq(i,j)*fq(i,j)
       enddo
c**************Read and diagonalize G-matrix block*************
       call getname(name,l_name,nblok,nmol,cr,l_cr)
       call readg(cr,l_cr,i,nfq(i),w,nfc,nq(i),npom,ve)
       write(inp) i,nq(i),nfq(i),((w(k,l), l=1,nfq(i)), k=1,nq(i))
c**************Read a Z matrix block**********************************
c*Z matrix must be ordered in the descending order of force constants*
c*********************************************************************
       call getname(name,l_name,nblok,0,cr,l_cr)
       print *,'Reading '//cr(1:l_cr)//'.zmat'
       open(unit=7,file=cr(1:l_cr)//'.zmat',status='old',form=
     . 'unformatted')
       read(7) noz(i)
       read(7)(nr(i,l),nc(i,l),nfo(i,l),z(i,l),l=1,noz(i))
       close(unit=7)
       print *,'Done with block ',i
      enddo
      close(unit=9)
c************Initialize selection*******************************
      fzmin1=1.d38
      fzmin2=1.d38
      nch=0
      change=.true.
      open(unit=7,file=name(1:l_name)//'.selout',status='unknown')
      write(7,'(a)') '# Testing f.c. addition ...'
   10 rewind inp
      sing=.true.
      do i=1,nb
c************Ordering of Z-matrix elements***********************
       l=1
       nz(i)=0
   20  tmpin=in(nfo(i,l),ncc,nvar,nfc)
       if(tmpin) then
        nz(i)=nz(i)+1
        ndown(i,nz(i))=l
       endif
       l=l+1
       if(l.gt.noz(i)) goto 60
   30  if(nfo(i,l).eq.nfo(i,l-1)) then
        l=l+1
        if(l.gt.noz(i)) goto 60
        goto 30
       else
        if(tmpin) nup(i,nz(i))=l-1
       endif
       goto 20
   60  if(tmpin) nup(i,nz(i))=noz(i)
c**************Preparation of H0 matrix**********************
       do k=1,nq(i)
        do l=1,nq(i)
         g(k,l)=0.0
        enddo
       enddo
       if(nfixed.eq.0) then
        np=0
        do l=1,nfq(i)
         do k=1,l
           h0(np+k,i)=0.d0
         enddo
         np=np+l
        enddo
       endif
       do j=1,ndown(i,1)-1
        nrow=nr(i,j)
        ncol=nc(i,j)
        g(nrow,ncol)=g(nrow,ncol)+z(i,j)*fi(nfo(i,j))
        g(ncol,nrow)=g(nrow,ncol)
       enddo
       do k=1,nz(i)-1
        do j=nup(i,k)+1,ndown(i,k+1)-1
         nrow=nr(i,j)
         ncol=nc(i,j)
         g(nrow,ncol)=g(nrow,ncol)+z(i,j)*fi(nfo(i,j))
         g(ncol,nrow)=g(nrow,ncol)
        enddo
       enddo
       do j=nup(i,nz(i))+1,noz(i)
        nrow=nr(i,j)
        ncol=nc(i,j)
        g(nrow,ncol)=g(nrow,ncol)+z(i,j)*fi(nfo(i,j))
        g(ncol,nrow)=g(nrow,ncol)
       enddo
       read(inp) ii,nrw,ncw,((w(k,l), l=1,ncw), k=1,nrw)
       call dsymm('L','U',nq(i),nfq(i),1.0d0,g,nfc,w,nfc,0.d0,opt,nfc)
       call dgemm('T','N',nfq(i),nfq(i),nq(i),1.0d0,w,nfc,opt,nfc,0.d0,
     .             g,nfc)
       np=0
       do l=1,nfq(i)
        do k=1,l
          h0(np+k,i)=g(k,l)
        enddo
        np=np+l
       enddo
      enddo
c***************Check dimensions*******************************
      k=0
      do i=1,nb
       if(nz(i).eq.0) then
        print*,'no active force constants in block ',i
        stop 4
       endif
       k=k+nfq(i)
      enddo
      if(k.lt.nvar+1) then
       print *,'Total dimension of jacobian = ',k
       print *,'Max. # of variable force constants = ',nvar+1
       print *,'System is underdetermined. Increase number of'
       print *,'frequencies or fix more force constants.'
       stop 5
      endif
c***************Diagonalize******************************
      call hmtx(nb,f,ve)
      print *,'Weighted square deviation : ',f
c*************Perform SVD of the Jacobian**********************
      call jacobian(nb,nrow,nvar)
      call dgesvd('s','s',nrow,nvar,aa,jarow,a,u,jarow,
     $             w,nfc,work,lwork,info)
      if(info.ne.0) print *,'svd error=',info
      do i=1,nvar
       if(a(i).lt.1.0d-8) then
        print *,' Singular value # ',i,'=                 ',a(i)
        sing=.false.
       endif
      enddo
c**********Calculate vector u'b**************************
      znorm1=0.d0
      do k=1,nvar
       work(k)=0.0d0
       do l=1,nrow
        work(k)=work(k)+u(l,k)*b(l)
       enddo
       z2=work(k)*work(k)
       znorm1=znorm1+z2
      enddo
      fz=sqrt(f*f-znorm1)
      znorm1=sqrt(znorm1)
      if(change.and.nch.gt.0) then
       print *,'Force constant ',nfix(nch),' added.'
       write(7,*) nfix(nch),fz
      else 
       if(num.gt.0) then
        print *,'Force constant ',ncc1(num),' eliminated.'
        write(7,*) ncc1(num),fz
       endif
      endif
      print *,'Refinable part of vector b    : ',znorm1
      print *,'Unrefinable part of vector b    : ',fz
      if(change) then
       if(fz.lt.fzmin1.and.sing.and.nch.gt.0) then
        fzmin1=fz
        nmin1=nfix(nch)
       endif
      else
       if(fz.lt.fzmin2.and.num.gt.0) then
        fzmin2=fz
        nmin2=ncc(num)
       endif
      endif
      if(nch.ge.nfixed1+nvar1) goto 999
c************Obtain new set of force constants**************
      nch=nch+1
      if(nch.le.nfixed1) then
       change=.true.
       m=0
   80  m=m+1
       if(ncc1(m).lt.nfix(nch).and.m.le.nvar1) then
        ncc(m)=ncc1(m)
        goto 80
       endif
       ncc(m)=nfix(nch)
       nvar=nvar1+1
       do mm=m+1,nvar
        ncc(mm)=ncc1(mm-1)
       enddo
      else
       if(change) then
        write(7,'(a)') '# Testing f.c. elimination ...'
        if(elimin) goto 999
       endif
       change=.false.
       num=nch-nfixed1
       nvar=nvar1-1
       do m=1,num-1
        ncc(m)=ncc1(m)
       enddo
       do m=num+1,nvar1
        ncc(m-1)=ncc1(m)
       enddo
      endif
      go to 10
  555 format(7(1x,e10.3))
  999 write(7,*) '""""""""""""""""""""""""""""""""""""""""""""""""""""'
      print *, '""""""""""""""""""""""""""""""""""""""""""""""""""""""'
      if(nmin1.gt.0) then
      print *, 'Recommended f.c. to add:    ',nmin1,' u.p.b.=',fzmin1
      write(7,*) 'Recommended f.c. to add:    ',nmin1,' u.p.b.=',fzmin1
      else
       print *, 'Recommended f.c. to add:   none'
       write(7,*) 'Recommended f.c. to add:    none'       
      endif
      if(.not.change) then
       print *, 'Recommended f.c. to remove: ',nmin2,' u.p.b.=',fzmin2
      write(7,*) 'Recommended f.c. to remove: ',nmin2,' u.p.b.=',fzmin2
      endif
      close(unit=7)
      stop 1
c  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 9990 print *
      print *,'Input operation unsuccesful. Check the name and format'
      print *,'of the data file. Don"t input filename extension !'
      stop 2
c  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      end

      subroutine readg(cr,l_cr,i,nfq,w,ldw,ncoln,npom,ve)
      implicit real*8(a-h,o-z)
      include 'Sizes'
      character*20 cr
      logical ve
      dimension gg(nint,nint),eigval(nint),w(ldw,ncoln),work(lwork)
      common /hkmatrices/ nr(nbl,ndimz),nc(nbl,ndimz),z(nbl,ndimz),
     . ndown(nbl,nfc),nup(nbl,nfc)
c***************Read G-matrix block******************************
       print *,'Reading '//cr(1:l_cr)//'.gmat'
       open(unit=7,file=cr(1:l_cr)//'.gmat',status='old',form
     . ='unformatted')
       read(7) nel
       read(7) (nr(i,l),nc(i,l),z(i,l),l=1,nel)   
       close(unit=7)
c***************Initialize G-matrix******************************
       do k=1,ncoln
        do l=1,ncoln
         gg(k,l)=0.d0
        enddo
       enddo
       do l=1,nel
        gg(nr(i,l),nc(i,l))=z(i,l)
        gg(nc(i,l),nr(i,l))=z(i,l)
       enddo
c****************Diagonalize G-matrix***********************
       call dsyev('V','U',ncoln,gg,nint,eigval,work,lwork,info)
       if(ve) then
        print *,'  == READG == '
        print *,'  == DSYEV ==  Error code on return : ',info
        write(*,'(a,f6.0)')
     .          '   == DSYEV ==  Optimal value of lwork : ',work(1)
        print *,'  == DSYEV ==  Current value of lwork : ',lwork
       endif
c****************Calculate scaled eigenvectors***************
       nzero=0
       do k=1,ncoln
        kk=ncoln-k+1
        if(dabs(eigval(kk)).lt.5.0d-5) then
         nzero=nzero+1
         eigval(kk)=0.d0
        endif
        do j=1,ncoln
         w(j,k)=gg(j,kk)*dsqrt(eigval(kk))
        enddo
       enddo
       npom=ncoln-nzero
       if(npom.ne.nfq) then
        print *,'Number of frequencies for',i,' block does'
        print *,'not match the rank of G-matrix = ',npom
        stop 3
       endif
      end

      subroutine jacobian(nb,nrow,nvar)
      implicit real*8(a-h,o-z)
      include 'Sizes'
      parameter (inp=2)
      dimension ww(nint,nint),el(nint)
      common /qmat/ q(ndim,ndim,nbl),fq(nbl,ndim),nfq(nbl),nz(nbl),
     .        nzfq(nbl,ndim),nfo(nbl,ndimz)
     . /jacob/ aa(jarow,nfc),u(jarow,nfc),ncc(nfc),nfix(nfc)
     . /hkmatrices/ nr(nbl,ndimz),nc(nbl,ndimz),z(nbl,ndimz),
     . ndown(nbl,nfc),nup(nbl,nfc)
c*************Construct jacobian matrix and decompose it*****
      rewind inp
      nrow=0
      do i=1,nb
       read(inp) ii,nrw,ncw,((ww(k,l), l=1,ncw), k=1,nrw)
       ncol=0
       do j=1,nz(i)
        ncol=ncol+1
  110   if(ncc(ncol).eq.nfo(i,ndown(i,j))) then
         kkk=1
         do l=1,nfq(i)
          if(l.ne.nzfq(i,kkk)) then
           nql1=nfq(i)-l+1
           call dgemv('N',nrw,nfq(i),1.d0,ww,nint,
     .                 q(1,nql1,i),1,0.d0,el,1)
           r=0.d0
           do jj=ndown(i,j),nup(i,j)
            nnr=nr(i,jj)
            nnc=nc(i,jj)
            pom=el(nnr)*el(nnc)*z(i,jj)
            r=r+pom
            if(nnr.ne.nnc) r=r+pom
           enddo
c************Un-weighted Jacobian***********
c           aa(nrow+l,ncol)=r
c********************************************
           aa(nrow+l,ncol)=r/sqrt(fq(i,l))
          else
           kkk=kkk+1
           aa(nrow+l,ncol)=0.d0
          endif
         enddo
        else
         do l=nrow+1,nrow+nfq(i)
          aa(l,ncol)=0.0d0
         enddo
         ncol=ncol+1
         goto 110
        endif
       enddo
       if(ncol.lt.nvar) then
        do nn=ncol+1,nvar
         do l=nrow+1,nrow+nfq(i)
          aa(l,nn)=0.0d0
         enddo
        enddo
       endif
       nrow=nrow+nfq(i)
      enddo
      return
      end

      subroutine hkmat(i,k,ldw,ncw,ww,nhk,hk)
      implicit real*8(a-h,o-z)
      include 'Sizes'
c**************Calculate Hk matrices********************
      dimension ww(ldw,ncw),hk(nhk)
      common /hkmatrices/ nr(nbl,ndimz),nc(nbl,ndimz),z(nbl,ndimz),
     . ndown(nbl,nfc),nup(nbl,nfc)
      npos=0
      do n=1,ncw
       do m=1,n
        r=0.0d0
        do j=ndown(i,k),nup(i,k)
         r=r+ww(nr(i,j),m)*ww(nc(i,j),n)*z(i,j)
         if(nr(i,j).ne.nc(i,j)) r=r+ww(nc(i,j),m)*ww(nr(i,j),n)*z(i,j)
        enddo
        hk(npos+m)=r
       enddo
       npos=npos+n
      enddo
      end

      subroutine diagh(i,inp,eigval,ve)
      implicit real*8(a-h,o-z)
      include 'Sizes'
      logical ve
      dimension ww(nint,nint),h(nkdim),hk(nkdim),work(lwork),
     .eigval(ndim)
      equivalence (hk(1),work(1))
      common /hkmatrices/ nr(nbl,ndimz),nc(nbl,ndimz),z(nbl,ndimz),
     .                    ndown(nbl,nfc),nup(nbl,nfc)
     . /qmat/ q(ndim,ndim,nbl),fq(nbl,ndim),nfq(nbl),nz(nbl),
     .        nzfq(nbl,ndim),nfo(nbl,ndimz)
     . /hmat/ h0(nkdim,nbl),b(jarow),fi(nfc),fi0(nfc)
c*************Diagonalization procedure********************
   10 read(inp) iw,nrw,ncw,((ww(k,l), l=1,ncw), k=1,nrw)
      if(iw.ne.i) goto 10
      if(ve) print *,'  == DIAGH ==  Preparing matrix H'
      np=0
      do jj=1,nfq(i)
       do ii=1,jj
         h(np+ii)=h0(np+ii,i)
       enddo
       np=np+jj
      enddo
      nhk=nfq(i)*(nfq(i)+1)/2
      do j=1,nz(i)
       fc=fi(nfo(i,ndown(i,j)))
       if(fc.ne.0.d0) then
        call hkmat(i,j,nint,ncw,ww,nhk,hk)
        np=0
        do jj=1,nfq(i)
         do ii=1,jj
           h(np+ii)=h(np+ii)+fc*hk(np+ii)
         enddo
         np=np+jj
        enddo
       endif
      enddo
      if(ve) print *,'  == DIAGH ==  Begin diagonalization'
      call dspev('V','U',nfq(i),h,eigval,q(1,1,i),ndim,work,info)
      if(ve) then
       print *,'  == DSPEV ==  Error code on return : ',info
       write(*,'(a,f6.0)')
     .         '   == DSPEV ==  Optimal value of lwork : ',work(1)
       print *,'  == DSPEV ==  Current value of lwork : ',lwork
      endif
      end

      subroutine hmtx(nb,fa,ve)
      implicit real*8(a-h,o-z)
      include 'Sizes'
      parameter (inp=2)
      dimension eigval(ndim)
      logical ve
      common /qmat/ q(ndim,ndim,nbl),fq(nbl,ndim),nfq(nbl),nz(nbl),
     .        nzfq(nbl,ndim),nfo(nbl,ndimz)
     . /hmat/ h0(nkdim,nbl),b(jarow),fi(nfc),fi0(nfc)
      f=0.0d0
      rewind inp
      nrow=0
      if(ve) print *,'  == HTMX ==  Start diagonalization'
      do i=1,nb
       call diagh(i,inp,eigval,ve)
c*************Construct b-vector**************************
       kk=1
       frob=0.0d0
       do j=1,nfq(i)
        nqj1=nfq(i)-j+1
        if(eigval(nqj1).lt.0.d0) print *,j,'-th eigenvalue < 0 !'
        if(j.ne.nzfq(i,kk)) then
c***************Un-weighted b-vector**************
c          p=fq(i,j)-eigval(nqj1)
c**************************************************
         p=(fq(i,j)-eigval(nqj1))/sqrt(fq(i,j))
         b(nrow+j)=p
         frob=frob+p*p
        else
         b(nrow+j)=0.0d0
         kk=kk+1
        endif
       enddo
       f=f+frob
       nrow=nrow+nfq(i)
      enddo
      fa=sqrt(f)
      return
      end
