C**********************************************************************
C                 Copyright 1992-1996, Robert Fraczkiewicz            *
C                           All rights reserved.                      *
C**********************************************************************
C This program is free software; you can redistribute it and/or modify*
C it under the terms of the GNU General Public License as published by*
C the Free Software Foundation; either version 1, or (at your option) *
C any later version.                                                  *
C                                                                     *
C This program is distributed in the hope that it will be useful,     *
C but WITHOUT ANY WARRANTY; without even the implied warranty of      *
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU    *
C General Public License for more details.                            *
C                                                                     *
C You should  have received a copy of the GNU General Public License  *
C along with this program; if not, write to the Free Software         *
C Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.           *
c*********************************************************************
c                                                                    *
c         Program "Rank of H-matrix" by Robert Fraczkiewicz          *
c                         Jun 1995, version 4.1                      *
c                                                                    *
c*********************************************************************
c Program calculates the rank of a H-matrix blocks. The Gaussian     *
c elimination method is used to find all linear dependencies among   *
c force constants.                                                   *
c*********************************************************************
c                                                                    *
c      INPUT FILES:                                                  *
c                                                                    *
c      ".rein" - main input file containing force field and obseved  *
c               vibrational frequencies                              *
c      ".zmat" - Z-matrix block in binary format (ZSYM4 output)      *
c      ".gmat" - G-matrix (output of GMAT4)                          *
c                                                                    *
c*********************************************************************
c                                                                    *
c      OUTPUT FILES:                                                 *
c                                                                    *
c      ".tout" - contains rank value and linear dependence relations *
c                                                                    *
c*********************************************************************
c                                                                    *
c      DIMENSIONS:                                                   *
c                                                                    *
c      nr,nc,z,nfo                    - NDIMZ                        *
c      fq                             - NDIM                         *
c      ncc,nonz,a,fi                  - NFC                          *
c      g,w                            - NDIMxNDIM                    *
c      ak,u                           - NAKxNFC                      *
c                                                                    *
c*********************************************************************
      program tmat
      implicit real*8(a-h,o-z)
      include 'Sizes'
      integer nak
      parameter(nak=2601,tol=1.d-4)
      character*20 name,cr
      character*80 comment
      logical change,all,ve
      dimension nfo(ndimz),fi(nfc),
     1w(ndim,ndim),g(ndim,ndim),ncc(nfc),a(nfc),u(nak,nfc),
     2fq(ndim),ak(nak,nfc),nonz(nfc)
      common /hkmatrices/ nr(ndimz),nc(ndimz),z(ndimz)
      equivalence (fq(1),g(1,1),ak(1,1),u(1,1))
c**********************************************************************
      if(iargc().ne.1) then
       print *,'^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^'
       print *
       print *,'                     T  M  A  T   '
       print *
       print *,'                     Version 4.1'
       print *
       print *,'                 Rank of a H-matrix '
       print *
       print *,'______________________________________________________'
       print *
       print *,'SYNOPSIS:'
       print *
       print *,'     tmat filename'
       print *
       print *,'PARAMETERS:'
       print *
       print *,'      filename - core name of the input and output'
       print *,'                 files. Its length cannot exceed 14'
       print *,'                 characters. Appropriate extensions'
       print *,'                 are added by default.'
       print *
       print *,'______________________________________________________'
       stop 1
      endif
      call getarg(1,name)
      l_name=namlen(name)
c**************Read initial set of force constants*****************
      open(unit=9,file=name(1:l_name)//'.rein',err=9990,status='old') 
      read(9,*) nf
      read(9,*) (fi(i),i=1,nf)
      read(9,*) nvar
      if(nvar.lt.0) then
       nvar=nf+nvar
       nfixed=nf-nvar
       read(9,*) (nr(i),i=1,nfixed)
       l=1
       j=1
       i=0
       do while(l.le.nfixed.or.j.le.nvar)
        i=i+1
        if(i.eq.nr(l)) then
         l=l+1
        else
         ncc(j)=i
         j=j+1
        endif
       enddo
      else
       nfixed=nf-nvar
       if(nfixed.eq.0) then
        do i=1,nvar
         ncc(i)=i
        enddo
       else
        read(9,*) (ncc(i), i=1,nvar)
       endif
      endif 
      read(9,*) nb
c**************Read experimental frequencies*******************
c nblok - number of symmetry block, nmol - number of molecule *
c**************************************************************
      do izz=1,nb
       read(9,'(i2,1x,i2,1x,a80)') nblok,nmol,comment
       read(9,*) nfq,nq
       read(9,*) (fq(j), j=1,nfq)
c**************Read and diagonalize a G-matrix block************
       call getname(name,l_name,nblok,nmol,cr,l_cr)
       call readg(cr,l_cr,izz,nfq,w,nq,npom,ve)
c**************Read a Z matrix block**********************************
c*Z matrix must be ordered in the descending order of force constants*
c*********************************************************************
       call getname(name,l_name,nblok,0,cr,l_cr)
       print *,'Reading '//cr(1:l_cr)//'.zmat'
       open(unit=7, file=cr(1:l_cr)//'.zmat', status='old',form=
     . 'unformatted')
       read(7) noz
       read(7)(nr(l),nc(l),nfo(l),z(l),l=1,noz)
       close(unit=7)
c**********Calculate the combined T-matrix*********************
       open(unit=16,file=cr(1:l_cr)//'.tout',status='unknown')
       nn=nfq*nfq
       if(nn.gt.nak) then
        print *,'The dimension of ',nblok,' - th block is too high'
        print *,nn,'>',nak
        stop 4
       endif
       ll=0
       do k=1,nvar
        do jj=1,nn
         ak(jj,k)=0.0d0
        enddo
        do l=ll+1,noz
         if(ncc(k).eq.nfo(l)) then
          ll=l
          j=nr(l)
          m=nc(l)
          do n1=1,nfq
           do n2=1,nfq
            ncol=nfq*(n1-1)+n2
            ak(ncol,k)=ak(ncol,k)+w(j,n1)*w(m,n2)*z(l)
            if(j.ne.m) ak(ncol,k)=ak(ncol,k)+w(m,n1)*
     1                w(j,n2)*z(l)
           enddo
          enddo
         endif         
        enddo
       enddo
       nonzero=0
       do k=1,nvar
        nc(k)=ncc(k)
        all=.true.
        ll=1
        do l=1,nn
         if(abs(ak(l,k)).gt.1.0d-8) then
          all=.false.
          nonzero=nonzero+1
          nonz(nonzero)=l
          nc(nonzero)=nc(k)
          do m=1,nn
           ak(m,nonzero)=ak(m,k)
          enddo
          go to 39
         endif
        enddo
   39   if(all) then
         write(16,41) nc(k)
        endif
       enddo
   41  format(1x,i3,' - th force constant not present')
       write(16,44) nvar-nonzero
   44  format(1x,i3,' force constants absent')
       npres=0
       do k=1,nonzero
        if(nonz(k).gt.0) then
         npres=npres+1
         write(16,46) nc(k)
         do i=k+1,nonzero
          if(nonz(i).eq.nonz(k)) then
           ii=nonz(i)
           ratio=ak(ii,i)/ak(ii,k)
           all=.true.
           do j=ii+1,nn
            if(ak(j,k).eq.0.0d0) then
             if(ak(j,k).ne.ak(j,i)) all=.false.
            else
             r=ak(j,i)/ak(j,k)-ratio
             if(abs(r).gt.1.0d-8) all=.false.
            endif
           enddo
           if(all) then
            nonz(i)=0
            if(ratio.gt.0.0) then
             write(16,42) nc(k),ratio,nc(i)
            else
             write(16,43) nc(k),ratio,nc(i)
            endif             
           endif
          endif
         enddo
        endif
       enddo
   46  format(1x,'Present : k(',i3,')')
   42  format(1x,'Linear dependence: k(',i3,')+',e10.4,'*k(',
     1i3,')')
   43  format(1x,'Linear dependence: k(',i3,') ',e10.4,'*k(',
     1i3,')')
   56  format(4(1x,i3,1x,i3,1x,e10.4))
       print *,'Writing '//cr(1:l_cr)//'.tout'
       write(16,45) comment,npres
      enddo
   57 format(5(1x,i3,1x,f10.6))
   45 format(1x,a80,/1x,i3,' force constants present')
   47 format(6(1x,f11.8))
      close(unit=9)
      close(unit=16)
      stop 0
c  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 9990 print *
      print *,'Input operation unsuccesful. Check the name and format'
      print *,'of the data file. don"t input extension of filename !'
      stop 2
c  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      end

      subroutine readg(cr,l_cr,i,nfq,w,ncoln,npom,ve)
      implicit real*8(a-h,o-z)
      include 'Sizes'
      character*20 cr
      logical ve
      dimension gg(ndim,ndim),eigval(ndim),w(ndim,ncoln),work(lwork)
      common /hkmatrices/ nr(ndimz),nc(ndimz),z(ndimz)
c***************Read G-matrix block******************************
       print *,'Reading '//cr(1:l_cr)//'.gmat'
       open(unit=7,file=cr(1:l_cr)//'.gmat',status='old',form
     . ='unformatted')
       read(7) nel
       read(7) (nr(l),nc(l),z(l),l=1,nel)   
       close(unit=7)
c***************Initialize G-matrix******************************
       do k=1,ncoln
        do l=1,ncoln
         gg(k,l)=0.d0
        enddo
       enddo
       do l=1,nel
        gg(nr(l),nc(l))=z(l)
        gg(nc(l),nr(l))=z(l)
       enddo
c****************Diagonalize G matrix***********************
       call dsyev('V','U',ncoln,gg,ndim,eigval,work,lwork,info)
       if(ve) then
        print *,'  == READG == '
        print *,'  == DSYEV ==  Error code on return : ',info
        write(*,'(a,f6.0)')
     .          '   == DSYEV ==  Optimal value of lwork : ',work(1)
        print *,'  == DSYEV ==  Current value of lwork : ',lwork
       endif
c****************Calculate scaled eigenvectors***************
       nzero=0
       do k=1,ncoln
        kk=ncoln-k+1
        if(dabs(eigval(kk)).lt.5.0d-5) then
         nzero=nzero+1
         eigval(kk)=0.d0
        endif
        do j=1,ncoln
         w(j,k)=gg(j,kk)*dsqrt(eigval(kk))
        enddo
       enddo
       npom=ncoln-nzero
       if(npom.ne.nfq) then
        print *,'Number of frequencies for',i,' block does'
        print *,'not match the rank of G-matrix = ',npom
        stop 3
       endif
      end
