C**********************************************************************
C                 Copyright 1992-1996, Robert Fraczkiewicz            *
C                           All rights reserved.                      *
C**********************************************************************
C This program is free software; you can redistribute it and/or modify*
C it under the terms of the GNU General Public License as published by*
C the Free Software Foundation; either version 1, or (at your option) *
C any later version.                                                  *
C                                                                     *
C This program is distributed in the hope that it will be useful,     *
C but WITHOUT ANY WARRANTY; without even the implied warranty of      *
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU    *
C General Public License for more details.                            *
C                                                                     *
C You should  have received a copy of the GNU General Public License  *
C along with this program; if not, write to the Free Software         *
C Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.           *
c**********************************************************************
c                                                                     *
c      Program "Z-Matrix Symmetrizer" by Robert Fraczkiewicz          *
c                         Jun 94, version 4.7                         *
c                                                                     *
c**********************************************************************
c This program symmetrizes Z-matrix. Z-matrix encodes F-matrix in     *
c terms of general valence force field parameters. It is specified by *
c a 3D array, z(i,j,k), where:                                        *
c        i is row number of the F-matrix element                      *
c        j is column number of the F-matrix element                   *
c        k is the number of an independent force constant.            *
c An element F(i,j) is defined as z(i,j,k)*(value of force constant). *
c The column number must be ALWAYS greater or equal to the row number.*
c The appropriate U-matrix filename must appear in the first line of  *
c the ".zsin" file.                                                   *
c**********************************************************************
c                                                                     *
c     INPUT FILES:                                                    *
c                                                                     *
c     ".zsin" - main input file containing user-prepared unsymmetrized*
c               Z-matrix in internal coordinates                      *
c     ".umat" - U-matrix (user-defined)                               *
c                                                                     *
c**********************************************************************
c                                                                     *
c     OUTPUT FILES:                                                   *
c                                                                     *
c     ".zsout" - main output containing Z matrix in ASCII format.     *
c                Not essential for other programs.                    *
c     ".zmat" - Z-matrices in binary format. Input for other programs *
c               in the package.                                       *
c                                                                     *
c**********************************************************************
c                                                                     *
c           Dimensions                                                *
c                                                                     *
c     nr,nc,nfo,z                                - NDIMZ*NBL          *
c     nrs,ncs,nfos,zs                            - NBLxNDIMZ          *
c     u                                          - INTERxINTER        *
c     nb,nozs                                    - NBL                *
c                                                                     *
c**********************************************************************
      implicit real*8(a-h,o-z)
      include 'Sizes'
      integer ndimz2
      parameter (ndimz2=nbl*ndimz,zero=0.d0,zeps=1.d-12)
      character*20 name,qs,cr
      character*80 record(3)*80
      logical change
      data qs /'?'/
      dimension  nr(ndimz2),nc(ndimz2),nfo(ndimz2),z(ndimz2),
     . nozs(nbl),nrs(nbl,ndimz),ncs(nbl,ndimz),nfos(nbl,ndimz),
     . zs(nbl,ndimz),u(inter,inter)
      common nsb,nb(nbl)
c**********************************************************************
      if(iargc().eq.0) then
       print *
       print *,' /--------------------------------------------------\ '
       print *,'|   ZZZZZZZ   SSSSS   Y     Y  M     M     4         |'
       print *,'|        Z   S     S   Y   Y   MM   MM    4          |'
       print *,'|       Z    S          Y Y    M M M M   4  4        |'
       print *,'|      Z      SSSSS      Y     M  M  M  4444444      |'
       print *,'|     Z            S     Y     M     M      4        |'
       print *,'|    Z       S     S     Y     M     M      4        |'
       print *,'|   ZZZZZZZ   SSSSS      Y     M     M      4        |'
       print *,'|                                                    |'
       print *,'|                     version 4.5                    |'
       print *,'|                                                    |'
       print *,'|             Z-matrix Symmetryzator                 |'
       print *,'|                                                    |'
       print *,'|                                                    |'
       print *,'|                           by Robert Fraczkiewicz   |'
       print *,'|                      University of Houston, 1992-4 |'
       print *,' \--------------------------------------------------/ '
       print *
       print *,'______________________________________________________'
       print *
       print *,'SYNOPSIS: '
       print *
       print *,'     zsym4 filename'
       print *
       print *,'PARAMETERS: '
       print *
       print *,'      filename - core name of the input and output'
       print *,'                 files. Its length cannot exceed 14'
       print *,'                 characters. Appropriate extensions'
       print *,'                 are added by default.'
       print *
       print *,'DESCRIPTION:'
       print *
       print *,' This program symmetrizes Z-matrix. Z-matrix encodes'
       print *,' F-matrix in terms of general valence force field'
       print *,' parameters. It is specified by a 3D array, z(i,j,k),'
       print *,' where:'
       print *,'     i is row number of the F-matrix element '
       print *,'     j is column number of the F-matrix element'
       print *,'     k is the number of an independent force constant.'
       print *,' An element F(i,j) is defined as z(i,j,k)*(value of'
       print *,' force constant).'
       print *,' The column number must be ALWAYS greater or equal to'
       print *,' the row number. The appropriate U-matrix filename'
       print *,' must appear in the first line of the ".zsin" file.'
       print *,' To see format of the ".zsin" file type "zsym4 \?".'
       print *
       stop 9
      endif
      call getarg(1,name)
      if (name.eq.qs) goto 9000
      l_name=namlen(name)
      open(unit=7, file=name(1:l_name)//'.zsin', err=9990, status='old') 
c***********************************************************************
c     Read U-matrix filename                                           *
c***********************************************************************
      read(7,'(a14)') qs
      l_qs=namlen(qs)
c***********************************************************************
c     Read problem control line containing the following information:  *
c       1. nq, the number of internal coordinates.                     *
c       2. nf, the number of force constants                           *
c***********************************************************************
      read(7,*) nq,nf
c***********************************************************************
c     Input data follows in order:                                     *
c      1. two problem information lines, format(a80)                   *
c      2. z-matrix elements in ascending order of force constants,     *
c      (warning !!! program does reindexing of force constants !!)     *
c         format(4(3i3,f9.6))                                          *
c***********************************************************************
      read(7,'(a80)')(record(i),i=1,2)
c************************
c     Read z-matrix     *
c************************
      noz=0
      do i=1,nf
       read(7,*) npi,nel,(nr(noz+l),nc(noz+l),z(noz+l),l=1,nel)
       if(npi.ne.i) npi=i
       do j=noz+1,noz+nel
        nfo(j)=npi
        if(nr(j).gt.nq.or.nc(j).gt.nq.or.nr(j).gt.nc(j)) goto 600
       enddo
       noz=noz+nel
      enddo
      close(unit=7)
      nfo(noz+1)=0      
      open(unit=8, file=name(1:l_name)//'.zsin_',status='unknown') 
      write(8,'(a14)') qs
      write(8,'(2i4)') nq,nf
      write(8,'(a80)')(record(i),i=1,2)
      li=1
      lcurr=1
      do l=2,noz+1
       if(nfo(l).eq.nfo(lcurr)) then
        li=li+1
       else
        last=lcurr+li-1
        if(li.gt.4) then
         write(8,60)nfo(lcurr),li,(nr(m),nc(m),z(m),
     .                               m=lcurr,lcurr+3)
         lb=int(li/4)
         do il=2,lb
          l0=lcurr+4*(il-1)
          ll=lcurr+4*il-1
          write(8,70)(nr(m),nc(m),z(m),m=l0,ll)
         enddo
         write(8,70)(nr(m),nc(m),z(m),m=lcurr+4*lb,last)
        else
         write(8,60)nfo(lcurr),li,(nr(m),nc(m),z(m),m=lcurr,last)
        endif     
        li=1
        lcurr=l   
       endif
      enddo
      close(unit=8)
   60 format(1x,i3,1x,i3,4(1x,i3,1x,i3,1x,f7.4))
   70 format(8x,4(1x,i3,1x,i3,1x,f7.4))
      open(unit=8, file=name(1:l_name)//'.zsout',status='unknown') 
      write(8,50) noz,(record(i),i=1,2)
   50 format(1x,'symmetrization of z-matrix. noz=',i4//80('*')/
     *       2(' *',a80,/),80('*')//)
c******************************************************************
c                 read and normalize u-matrix                     *
c******************************************************************
      open(unit=3, file=qs(1:l_qs)//'.umat', err=9991, status='old') 
      read(3,*) nnt,ns,nsb
      read(3,*)(nb(i),i=1,nsb)
      if(nnt.ne.nq) goto 603
      do i=1,ns
       do j=1,nnt
        u(i,j)=zero
       enddo
      enddo
      do i=1,ns
       read(3,*) nzr,(ncl,u(i,ncl),l=1,nzr)
       if(ncl.gt.nq) goto 605
      enddo
      close(unit=3)
      do i=1,ns
       du=0.0d0
       do j=1,nnt
        du=du+u(i,j)*u(i,j)
       enddo
       du=sqrt(du)
       do j=1,nnt
        u(i,j)=u(i,j)/du
       enddo
      enddo
c***************************************************************
c               factor z-matrix                                *
c***************************************************************
      nsb1=nsb-1
      ncurr=nfo(1)
      do i=1,nsb1
       nozs(i)=0
      enddo
      li=0
      do l=1,noz
       if(nfo(l).eq.ncurr) then
        li=li+1
       else
        do i=1,ns
         do j=i,ns
          r=zero
          do m=l-li,l-1
           nnr=nr(m)
           nnc=nc(m)
           r=r+u(i,nnr)*u(j,nnc)*z(m)
           if(nnr.ne.nnc) r=r+u(i,nnc)*u(j,nnr)*z(m)
          enddo
          if(abs(r).gt.zeps) then
           ni=nrblock(i)
           nj=nrblock(j)
           if(ni.ne.nj) goto 607
           nozs(ni)=nozs(ni)+1
           nrs(ni,nozs(ni))=i-nb(ni)+1
           ncs(ni,nozs(ni))=j-nb(ni)+1
           nfos(ni,nozs(ni))=ncurr
           zs(ni,nozs(ni))=r           
          endif
         enddo
        enddo
        ncurr=nfo(l)
        li=1
       endif
      enddo
      do i=1,ns
       do j=i,ns
        r=zero
        do m=l-li,l-1
         nnr=nr(m)
         nnc=nc(m)
         r=r+u(i,nnr)*u(j,nnc)*z(m)
         if(nnr.ne.nnc) r=r+u(i,nnc)*u(j,nnr)*z(m)
        enddo
        if(abs(r).gt.zeps) then
         ni=nrblock(i)
         nj=nrblock(j)
         if(ni.ne.nj) goto 607
         nozs(ni)=nozs(ni)+1
         nrs(ni,nozs(ni))=i-nb(ni)+1
         ncs(ni,nozs(ni))=j-nb(ni)+1
         nfos(ni,nozs(ni))=ncurr
         zs(ni,nozs(ni))=r           
        endif
       enddo
      enddo
c****************************************************************
c                       write output                            *
c****************************************************************
      do nso=1,nsb1
       if(nso.le.9) then
        cr='_'//char(nso+48)
       else
        nin1=int(float(nso)/10.0)
        nin2=nso-10*nin1
        cr='_'//char(nin1+48)//char(nin2+48)
       endif
       l_cr=namlen(cr)
       open(unit=9, file=name(1:l_name)//cr(1:l_cr)//'.zmat', 
     .      status='unknown', form='unformatted')
       write(8,64) nso
   64  format(1x,'  block ',i3)
       write(8,17)(nrs(nso,l),ncs(nso,l),nfos(nso,l),zs(nso,l),
     .              l=1,nozs(nso))
       write(9) nozs(nso)
       write(9)(nrs(nso,l),ncs(nso,l),nfos(nso,l),zs(nso,l),
     .           l=1,nozs(nso))
       close(unit=9)
      enddo
   17 format(4(3i3,f11.8))
      close(unit=8)
      stop 1
  600 print *,'Error in '//name(1:l_name)//'.zsin'
      print *,'Inappropriate row or column number'
      print *,'Field : ',nfo(j),nr(j),nc(j),z(j)
      stop 5
  603 print *,'Error in '//name(1:l_name)//'.zsin or 
     . '//qs(1:l_qs)//'.umat'
      print *,'Internal coordinate dimensions are not equal'
      print *,'in '//name(1:l_name)//'.zsin = ',nq
      print *,'in '//qs(1:l_qs)//'.umat = ',nnt
      stop 6
  605 print *,'Error in '//qs(1:l_qs)//'.umat'
      print *,'Inappropriate column number'
  701 print *,'Column ',ncl,' # internal coords. ',nnt
      stop 7
  607 print *,'Z-matrix is not factoring !!'
      print 11,i,j,r,nfo(l-1)
   11 format(1x,'element z(',i3,',',i3,')=',d11.4,' f.c.=',i4)
      stop 8
c  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 9991 print *
      print *,'Input operation unsuccesful.'
      print *,'Cannot find : ',qs(1:l_qs)//'.umat'
      stop 2
 9990 print *
      print *,'Input operation unsuccesful. check the name and format'
      print *,'of the data file. Don"t input extension of filename !'
 9000 print *,'Data file format :'
      print *
      print *,'name of the u-matrix file'
      print *,'nq nf'
      print *,'(comment line)'
      print *,'(comment line)'
      print *,'npi nel nro nco dat ... nro nco dat'
      print *,'(.................................)'
      print *,'npi nel nro nco dat ... nro nco dat'
      print *
      print *,'where : nq - number of internal coords'
      print *,'nf - number of force constants, nro, nco - the row and'
      print *,'column number in the z-matrix ,npi - number of force  '
      print *,'constant, dat - z-matrix element, nel - number of ele-'
      print *,'ments.'
      print *
      stop 3
c  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      end

      integer function nrblock(k)
      include 'Sizes'
      common nsb,nb(nbl)
      do i=1,nsb-1
       if(k.ge.nb(i).and.k.lt.nb(i+1)) goto 10
      enddo
      print *,'Error - number of symmetry coordinate requested = ',k
      print *,'Highest row dimension of U-matrix = ',nb(nsb)-1
      stop 4
   10 nrblock=i
      return
      end

      integer function namlen(stri)
      implicit real*8(a-h,o-z)
      character*14 stri
      do i=1,len(stri)
       if(stri(i:i).eq.' ') goto 2
      enddo
    2 namlen=i-1
      return
      end
